@@ -56,8 +56,14 @@ recvClientSecondFlight13 sparams ctx (appKey, clientHandshakeSecret, authenticat
             expectFinished sparams ctx chExtensions appKey clientHandshakeSecret sfSentTime
     if not authenticated && serverWantClientCert sparams
         then runRecvHandshake13 $ do
-            recvHandshake13 ctx $ expectCertificate sparams ctx
-            recvHandshake13hash ctx "CertVerify" (expectCertVerify sparams ctx)
+            -- RFC 8446 Sec 4.4.3: Clients MUST send this message
+            -- whenever authenticating via a certificate (i.e., when the
+            -- Certificate message is non-empty).  When sent, this message MUST
+            -- appear immediately after the Certificate message and immediately
+            -- prior to the Finished message.
+            skip <- recvHandshake13 ctx $ expectCertificate sparams ctx
+            unless skip $
+                recvHandshake13hash ctx "CertVerify" (expectCertVerify sparams ctx)
             recvHandshake13hash ctx "Finished" expectFinished'
             ensureRecvComplete ctx
         else
@@ -105,19 +111,21 @@ expectEndOfEarlyData ctx clientHandshakeSecret EndOfEarlyData13 = do
 expectEndOfEarlyData _ _ hs = unexpected (show hs) (Just "end of early data")
 
 expectCertificate
-    :: MonadIO m => ServerParams -> Context -> Handshake13 -> m ()
+    :: MonadIO m => ServerParams -> Context -> Handshake13 -> m Bool
 expectCertificate sparams ctx (Certificate13 certCtx (CertificateChain_ certs) _ext) = liftIO $ do
     when (certCtx /= "") $
         throwCore $
             Error_Protocol "certificate request context MUST be empty" IllegalParameter
     -- fixme checking _ext
     clientCertificate sparams ctx certs
+    return $ isNullCertificateChain certs
 expectCertificate sparams ctx (CompressedCertificate13 certCtx (CertificateChain_ certs) _ext) = liftIO $ do
     when (certCtx /= "") $
         throwCore $
             Error_Protocol "certificate request context MUST be empty" IllegalParameter
     -- fixme checking _ext
     clientCertificate sparams ctx certs
+    return $ isNullCertificateChain certs
 expectCertificate _ _ hs = unexpected (show hs) (Just "certificate 13")
 
 sendNewSessionTicket
